/* cmdline.cc
 * driver for hmetis
 */

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

extern "C" {
    HMETIS_PartRecursive(int, int, int *, int *, int *, int *, int, int, int *, int *, int *);
}

int
main(int argc, char *argv[])
{

    if (argc != 10) {
	fprintf(stderr,
	    "%s hgraphfile nparts ubfactor nruns ctype rtype vcycle reconst dbglvl\n",
	    argv[0]);
        exit(1);
    }
    int nparts = atoi(argv[2]);
    int ubfactor = atoi(argv[3]);
    int options[8];
    options[0] = 1; // no defaults
    options[1] = atoi(argv[4]);
    options[2] = atoi(argv[5]);
    options[3] = atoi(argv[6]);
    options[4] = atoi(argv[7]);
    options[5] = atoi(argv[8]);
    options[6] = -1;
    options[7] = atoi(argv[9]);
    printf("nparts: %d, ubfactor: %d\n", nparts, ubfactor);

    /* read hypergraph into memory */
    FILE *fp = fopen(argv[1], "r");
    assert(fp);
    int nVts, nHdgs, format;
    int cnt = fscanf(fp, "%d %d %d\n", &nHdgs, &nVts, &format);
    if (cnt < 2) exit(1);
    if (cnt == 2) format = 0;
    const int BUFLEN = 65536;
    char buf[BUFLEN];

    // first, simply count the total # of vertices in all hyperedges (to size eind)
    int vtxcnt = 0; // total # of vertices
    for (int i = 0; i < nHdgs; i++) {
        fgets(buf, BUFLEN, fp);
	int pos = 0;
	int incr = 0;
	int dummy; // we don't care about the values
	if (format == 1 || format == 11) {
	    // scan the edge weight
	    sscanf(&(buf[pos]), "%d%n", &dummy, &incr);
	    pos += incr;
	}
	// scan each vertex on a hyperedge individually
	while (sscanf(&(buf[pos]), "%d%n", &dummy, &incr) >= 0) {
	    vtxcnt++;
	    pos += incr;
	}
    }
    // reopen file and reposition where edges start
    fclose(fp);
    fp = fopen(argv[1], "r");
    int dummy;
    fscanf(fp, "%d %d %d\n", &dummy, &dummy, &dummy);

    // initialize storage for edges, weights, etc.
    int* eptr = (int *) malloc((nHdgs+1) * sizeof(int));
    int *eind = (int *) malloc(vtxcnt * sizeof(int));
    int* hewgts = NULL;
    int* vwgts = NULL;
    if (format == 1 || format == 11) {
	// we have weights for edges
        hewgts = (int *) malloc(nHdgs * sizeof(int));
    }
    if (format == 10 || format == 11) {
        vwgts = (int *) malloc(nVts * sizeof(int));
    }

    vtxcnt = 0;
    for (int i = 0; i < nHdgs; i++) {
	fgets(buf, BUFLEN, fp);
	//printf("%s: %d\n", buf, strlen(buf));
	eptr[i] = vtxcnt;
	int pos = 0;
	int incr = 0;
	if (format == 1 || format == 11) {
	    // scan the edge weight
	    sscanf(&(buf[pos]), "%d%n", &hewgts[i], &incr);
	    pos += incr;
	}
	// scan each vertex on a hyperedge individually
	while (sscanf(&(buf[pos]), "%d%n", &(eind[vtxcnt]), &incr) >= 0) {
	    vtxcnt++;
	    pos += incr;
	}
	//printf("\n");
    }
    eptr[nHdgs] = vtxcnt;

    if (format == 10 || format == 11) {
        // scan the vertex weights
	for (int i = 0; i < nVts; i++) {
	    fgets(buf, BUFLEN, fp);
	    sscanf(buf, "%d\n", &vwgts[i]);
	}
    }

    fclose(fp);
    printf("scanned hgraph\n");

    int* part = (int *) malloc(nVts * sizeof(int));
    int edgecut;
    printf("PartRecursive(nVts = %d, nHdgs = %d)\n", nVts, nHdgs);
    int j = 0;
    for (int i = 0; i <= nHdgs; i++) {
	printf("%d\n", eptr[i]);
        while (j < eptr[i]) {
	    printf("%d ", eind[j]);
	    j++;
	}
	printf("\n");
    }
    printf("%d\n", eptr[nHdgs]);
    HMETIS_PartRecursive(nVts, nHdgs, vwgts, eptr, eind, hewgts, nparts, ubfactor, 
        options, part, &edgecut);
    
    // output
    char outfile[128];
    sprintf(outfile, "%s.part.%-d", argv[1], nparts);
    printf("outfile: %s, edgecut: %d\n", outfile, edgecut);
    FILE *out = fopen(outfile, "w");
    for (int i = 0; i < nVts; i++) {
        fprintf(out, "%d\n", part[i]);
    }
    fclose(out);
}
