// NodeView.java
// Copyright (c) 1998, Regents of the University of California
// $Id: NodeView.java,v 1.23 1998/12/22 02:00:10 mashah Exp $

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

class Rect {
  double x1, x2, y1, y2;
}

class Pt {
  double x, y;
}

class Circle {
  double x, y, r;
}

public class NodeView extends JInternalFrame implements 
InternalFrameListener 

{

public double minx, miny, maxx, maxy;
public Rect[]   rects;
public Pt[]     points;
public Circle[] circles;

public Node        node;

// public int highlights[] = new int[1];
// public int hlen = 0;

public int levels = 0;

class Inside extends JComponent {

public boolean refreshTree = true;
public boolean showSubtree = false;
public boolean showSplit = false;
public boolean showPath  = false;

// Path information stored in treeview
// Split information
public int splitItems[] = null;
public int numSplit = 0;

// Highlight subtree information
public int subtreeRoot;
public int subtreeLevels;

public Font font = null;
public int numPreds = 0;
// public TreeView tv;
public Color[] hcolors = { Color.black, Color.red, Color.green, Color.yellow };

public void paint(Graphics g) {	    
    
    int width  = getBounds().width;
    int height = getBounds().height;
    
    // DbgOutput.println(1,getBounds());
    
    try {
	// g.setColor(Color.black);
	if (font == null)
	    font = new Font("Dialog", Font.PLAIN, 10);

	g.setFont(font);

	if (refreshTree) {
	    if (showPath) 
		showPathChildInternal();
	    else if (showSplit)
		showSplitInternal(splitItems, numSplit);
	    else if (showSubtree)
		highlightSubtreeInternal(subtreeRoot, subtreeLevels);
	    else
		showSubtreeInternal(node.nodenum);
	}

// 	if (!showSplit)
// 	    Libgist.display(node.nodenum, highlights, hlen,
// 		g, Color.black, Color.red, width, height);
// 	else
// 	    Libgist.display(node.nodenum, splitItems, numSplit,
// 		g, Color.black, Color.red, width, height);

	// Clear the screen if not already cleared ...
	g.setColor(getBackground());
	g.fillRect(0,0,width,height);

	Libgist.displayPreds(g, width, height, hcolors, tv.predsA, 
 	    numPreds);

	refreshTree = true;

	// tv.predMutex = 0;

    } catch (LibgistException e) {
  	System.out.println("Libgist.displayPreds() messed up");
	System.out.println("Exception: " + e);
    }
}

}

public Inside can;

// Stuff from tree view ...
public TreeView tv;

public int path[];
public int pl;

public String getViewName() {
    
    return getTitle();
}

public void showPathChild() {

    can.showPath = true;

    can.refreshTree = true;

    // Turn off split visualization and subtree highlights ...
    can.showSplit = false;
    can.showSubtree = false;

}

public void showPathChildInternal() {

    // int child;
    int pindex = -1;

    int i,j;

    // get current subtree ...

    can.showPath = true;

    PredInfoPack pinfo;

    // This thing is only ever accessed by one thread ... I think ...
    // so this should be okay ...
    // Nope leads to deadlock ... sorry ...
//     while(tv.predMutex != 0)
//  	Thread.yield();
    
//     tv.predMutex = 1;

    pinfo = tv.getSubtree(tv.predsA, node.nodenum, levels);
    tv.predsA = pinfo.preds;
    can.numPreds = pinfo.size;
    
    while (can.numPreds + pl > tv.predsA.length)
	tv.predsA = tv.doublePredSize(tv.predsA);

    // For now find child ... but next time keep a path structure 
    // so you don't have to do this.
    for (i=0; i < pl; i++) {
	// DbgOutput.println(1,path[i]);
	if ((path[i] == node.nodenum) /* && (i != 0) */ ) {
	    // child = path[i-1];
	    pindex = i;
	}
    }

    for (i = 0, j = 0; (i < levels) && (pindex > 0); i++) {
	try {
	    tv.predsA[can.numPreds + i].setFields(path[pindex],
		    Libgist.getParentPos(path[pindex - 1]), 
		    Libgist.getLevel(path[pindex]), 1);
	    //		child = path[pindex - 1];
	    j++;
	    pindex--;
	} catch (LibgistException ex) {
	    System.out.println("Error getting parent position: " + 
		path[pindex] + " : " +  ex);
	}
    }

    can.numPreds += j;

//     if (child != -1) {
// 	try {
	    
// 	    highlights[0] = Libgist.getParentPos(child);
// 	    hlen = 1;
// 	} catch (LibgistException ex) {
// 	    System.out.println("Error getting parent position: " + child + ex);
// 	}
//     } else
// 	hlen = 0;

    can.refreshTree = false;

    // Turn off split visualization and subtree highlights ...
    can.showSplit = false;
    can.showSubtree = false;
}

public void showSplit(int splitItems[], int num) {
    
    can.splitItems = splitItems;
    can.numSplit = num;

    can.showSplit = true;

    can.refreshTree = true;

    // Turn off show subTree ...
    can.showSubtree = false;
    can.showPath = false;
}

public void showSplitInternal(int splitItems[], int num) {

    // adjust DisplayPredInfo here ...
    
    // can.numPreds = tv.getSubtree(node.nodenum);
    
    int i;

    can.splitItems = splitItems;
    can.numSplit = num;

    can.showSplit = true;

    PredInfoPack pinfo;

//     while(tv.predMutex != 0)
//  	Thread.yield();
    
//     tv.predMutex = 1;

    pinfo = tv.getSubtree(tv.predsA,node.nodenum, levels);
    tv.predsA = pinfo.preds;
    can.numPreds = pinfo.size;
    
    while (can.numPreds + can.numSplit > tv.predsA.length)
	tv.predsA = tv.doublePredSize(tv.predsA);

    // Add split items to bottom

    for (i = can.numPreds; i < can.numPreds + can.numSplit; i++) {
	tv.predsA[i].setFields(node.nodenum,
	    can.splitItems[i - can.numPreds], 
	    Libgist.getLevel(node.nodenum), 3);
// 	tv.predsA[i].node = node.nodenum;
// 	tv.predsA[i].slot = can.splitItems[i - can.numPreds];
// 	tv.predsA[i].color = 1;
    }

    can.numPreds += can.numSplit;

    can.refreshTree = false;

    // Turn off show subTree ...
    can.showSubtree = false;
    can.showPath = false;
}

public void highlightSubtree(int num, int nlevels) {

    can.showSubtree = true;

    can.subtreeRoot = num;
    can.subtreeLevels = nlevels;

    can.refreshTree = true;

    // Gotta turn off split visualization ...
    can.showSplit = false;
    can.showPath = false;
}

public void highlightSubtreeInternal(int num, int nlevels) {

    DisplayPredInfo[] temp;

    int numhigh, i;
    
    // DbgOutput.println(1,"Can't show subtree right now ... ");
    can.showSubtree = true;

//     while(tv.predMutex != 0)
//  	Thread.yield();
    
//     tv.predMutex = 1;


    can.subtreeRoot = num;
    can.subtreeLevels = nlevels;

    // adjust the DisplayPredInfo here ...
    PredInfoPack pinfo = tv.getSubtree(tv.predsA,node.nodenum,levels);
    tv.predsA = pinfo.preds;
    can.numPreds = pinfo.size;

    pinfo = tv.getSubtree(tv.predsB, num, nlevels);
    tv.predsB = pinfo.preds;
    numhigh = pinfo.size;

    // make sure there is enough space for highlighted items ...
    while (can.numPreds + numhigh + 1 > tv.predsA.length)
	tv.predsA = tv.doublePredSize(tv.predsA);

    for (i = can.numPreds; i < can.numPreds + numhigh; i++) {
	tv.predsA[i].copyFields(tv.predsB[i - can.numPreds]);
	tv.predsA[i].color = 2;
    }


    try {

	if (num != node.nodenum) {
	    int tnum = Libgist.getParent(num);
	    tv.predsA[i].setFields(tnum,
		Libgist.getParentPos(num), Libgist.getLevel(tnum), 2);
	    can.numPreds += numhigh + 1;
	} else
	    can.numPreds += numhigh;
    } catch (Exception ex) {
	System.out.println("Hey: " + ex);
    }

    can.refreshTree = false;

    // Gotta turn off split visualization ...
    can.showSplit = false;
    can.showPath = false;
}

public void showSubtree(int num) {

    can.refreshTree = true;

    can.showPath = false;
    can.showSplit = false;
    can.showSubtree = false;
}

public void showSubtreeInternal(int num) {

//     while(tv.predMutex != 0)
//  	Thread.yield();
    
//     tv.predMutex = 1;

    PredInfoPack pinfo = tv.getSubtree(tv.predsA,num,levels);
    tv.predsA = pinfo.preds;
    can.numPreds = pinfo.size;

    can.refreshTree = false;

    can.showPath = false;
    can.showSplit = false;
    can.showSubtree = false;
}

public NodeView(Node node, TreeView tv, int levels) {
    
    this.node = node;

    if (levels == 0) {
	setTitle("Subtree View: " + node.nodenum + " : all");
	this.levels = Libgist.getLevel(node.nodenum);
    }
    else {
	setTitle("Subtree View: " + node.nodenum + " : " + levels);
	this.levels = levels;
    }

    setSize(150,150);

    Rectangle tbounds = tv.getBounds();
    Point location = new Point();
    location.x = tbounds.x;
    location.y = tbounds.y + tbounds.height;
    
    setLocation(location);

    setResizable(true);
    setMaximizable(true);
    setIconifiable(true);
    setClosable(true);
    getContentPane().setLayout(new BorderLayout());

    // genPtsnRects(250,2550,250,2550,120);

    can = new Inside();

    path = tv.path;
    pl   = tv.pl;

    // Argh ... this is messy ... will clean later ..
    this.tv = tv;
    // can.tv = tv;

    // showSubtree(node.nodenum);

    // add myself to the node 
    tv.nodeviews.addElement(this);    
    
    showPathChild();

    can.repaint();

    getContentPane().add(can, BorderLayout.CENTER);

    addInternalFrameListener(this);
}

public void genPtsnRects(double x1, double x2, double y1, double y2, int num) {

    // DbgOutput.println(1,"gen : " + x1 + " " + y1 + " " + x2 + " " + y2);
    double rangex = x2 - x1;
    double rangey = y2 - y1;

    points  = new Pt[num];
    rects   = new Rect[num];
    circles = new Circle[num];

    Random random = new Random();

    int i;

    double t1, t2;

    for (i=0; i < num; i++) {

	points[i] = new Pt();
	points[i].x = (random.nextDouble() * rangex) + x1;
	points[i].y = (random.nextDouble() * rangey) + y1;

	rects[i] = new Rect();
	rects[i].x1 = (random.nextDouble() * rangex) + x1;
	rects[i].x2 = (random.nextDouble() * rangex) + x1;
	rects[i].y1 = (random.nextDouble() * rangex) + y1;
	rects[i].y2 = (random.nextDouble() * rangex) + y1;

	circles[i] = new Circle();
	t1 = circles[i].x = (random.nextDouble() * rangex) + x1;
	t2 = circles[i].y = (random.nextDouble() * rangey) + y1;
	t1 = Math.min(Math.abs(x2 - t1),Math.abs(x1 - t1));
	t2 = Math.min(Math.abs(y2 - t2),Math.abs(y1 - t2));
	t1 = Math.min(t1,t2);
	circles[i].r = (random.nextDouble() * t1);
    }
    
    minx = x1;

    miny = y1;

    maxx = x2;
    
    maxy = y2;
}

public void paintStuff(int nodenum, Graphics g, int width, int height) {
    double rangex = maxx - minx + 1;
    double rangey = maxy - miny + 1;
    
    double xr = (double) width / rangex;
    double yr = (double) height / rangey;
    
    int i;
    double x,y,x2,y2;
    
    g.setColor(Color.red);
    // draw Rectangles ...
    for (i = 0; i < rects.length; i++) {
	x = Math.floor((rects[i].x1 - minx) * xr);
	x2 = Math.ceil((rects[i].x2 - minx) * xr);
	y = Math.floor((rects[i].y1 - miny) * yr);
	y2 = Math.ceil((rects[i].y2 - miny) * yr);
	
	// DbgOutput.println(1,x + " " + y + " " + x2 + " " + y2);
	//DbgOutput.println(1,rects[i].x1 + " " + rects[i].x2 + " " +
	// rects[i].y1 + " " + rects[i].y2);

	g.drawRect((int)x,(int)y,(int)(x2-x),(int)(y2-y));
    }
    
    g.setColor(Color.blue);
    for (i = 0; i < points.length; i++) {
	x = Math.floor((points[i].x - minx) * xr);
	y = Math.floor((points[i].y - miny) * yr);
	
	g.drawRect((int)x,(int)y,1,1);
    }

    double w, h;

    g.setColor(Color.green);
    for (i = 0; i < circles.length; i++) {
	x = Math.floor((circles[i].x - minx) * xr);
	y = Math.floor((circles[i].y - miny) * yr);
	
	w = Math.floor((circles[i].r) * xr);
	h = Math.floor((circles[i].r) * yr);

	g.drawOval((int) (x - w), (int) (y - h), (int) (2*w), (int) (2*h));
    }
}

public void internalFrameActivated(InternalFrameEvent ie) {};

public void internalFrameClosed(InternalFrameEvent ie) {
    // DbgOutput.println(1,"THIS");
};

public void internalFrameClosing(InternalFrameEvent ie) {
    // DbgOutput.println(1,"WHAT");
    // setVisible(false);
}

public void internalFrameDeactivated(InternalFrameEvent ie) {};

public void internalFrameDeiconified(InternalFrameEvent ie) {};

public void internalFrameIconified(InternalFrameEvent ie) {};

public void internalFrameOpened(InternalFrameEvent ie) {};

}

